//three.js（r73）対応
var GRAPH = {};


////////////////////////////////////////////////////////////////////
// ２次元カラーマップクラスの定義
////////////////////////////////////////////////////////////////////
GRAPH.Colormap2D = function( parameter ){
	parameter = parameter || {};

	//格子点数
	this.N = parameter.N || 100;
	//格子間隔
	this.l = parameter.l || 1.0 / this.N;

	//場の量の最小値と最大値
	this.min = (parameter.min !== undefined)? parameter.min : 0;   //最小値
	this.max = (parameter.max !== undefined)? parameter.max : 1;   //最大値

	//色関連パラメータ
	var p = parameter.colorParameter || {};
	this.colorParameter = {
		HSLMode : p.HSLMode || false,             //HSLモード実行のフラグ
		H0 : (p.H0 !== undefined )? p.H0 : 0.70,  //Hの始点（HSLモード時）
		Hw : (p.Hw !== undefined )? p.Hw : 0.73,  //Hの変化（HSLモード時）
		S : (p.S !== undefined )? p.S : 1,        //Sの値（HSLモード時）
		L : (p.L !== undefined )? p.L : 0.5,      //Lの値（HSLモード時）
		backWhite : p.backWhite || false          //中間値を白とするフラグ（RGBモード時）
	};

	var p = parameter.transparentMode || {};

	//透過モードの設定
	this.transparentMode = {
		enabled : p.enabled || false, //透過利用の有無
		factor : p.factor || 1,       //透明度因子
	};

	var p = parameter.checkeredPatternMode || {};

	//市松模様モードの設定
	this.checkeredPatternMode = {
		enabled : p.enabled || false, //市松模様利用の有無
		width : p.width || 5,         //市松模様の格子間隔
		L : p.L || 0.4                //市松模様暗部のHSLのL値
	};

	//場の値（２重配列）
	this.fieldValues = parameter.fieldValues || [];

	//コンピュータ・グラフィックスオブジェクト
	this.CG = null;

	//カラーマップの初期化
	this.initColormap();
}

//２次元カラーマップを実現するために必要な初期設定を行う
GRAPH.Colormap2D.prototype.initColormap = function(){

	//透過モード利用の有無
	if( this.transparentMode.enabled ){

		//テクスチャオブジェクトの生成
		var texture = new THREE.Texture( this.generateCanvas() );

		//形状オブジェクトの宣言と生成
		var geometry = new THREE.PlaneBufferGeometry( 1, 1 );

		//材質オブジェクトの宣言と生成
		var material = new THREE.MeshBasicMaterial({ 
			map : texture,                    //テクスチャ
			transparent : true,               //透過の有無
			depthTest : false,                //デプステストの有無
			blending : THREE.AdditiveBlending //ブレンディング方法の指定
		});

		//２次元格子オブジェクトの生成
		this.CG = new THREE.Mesh( geometry, material );

	} else {

		//形状オブジェクトの宣言と生成
		var geometry = new THREE.Geometry();
		//頂点座標の準備
		for (var i = 0; i <= this.N; i++) {
			for (var j = 0; j <= this.N; j++) {
				var nx = (-this.N / 2 + i) * this.l;
				var ny = (-this.N / 2 + j) * this.l;
				var nz = 0;
				//頂点座標データの追加
				geometry.vertices.push( new THREE.Vector3(nx, ny, nz) );
			}
		}

		//面指定配列の準備
		for (var i = 0; i < this.N; i++) {
			for (var j = 0; j < this.N; j++) {
				//頂点番号
				var ii = (this.N + 1) * i + j;
				//面指定用頂点インデックスを追加
				geometry.faces.push( new THREE.Face3( ii, ii + (this.N + 1), ii + (this.N + 1) + 1) );
				//面指定用頂点インデックスを追加
				geometry.faces.push( new THREE.Face3( ii, ii + (this.N + 1) + 1, ii + 1 ) );
				//頂点番号
				var n = 2 * (this.N * i + j);

				//頂点色の初期化
				geometry.faces[n].vertexColors[0] = new THREE.Color();
				geometry.faces[n].vertexColors[1] = new THREE.Color();
				geometry.faces[n].vertexColors[2] = new THREE.Color();
				geometry.faces[n+1].vertexColors[0] = new THREE.Color();
				geometry.faces[n+1].vertexColors[1] = new THREE.Color();
				geometry.faces[n+1].vertexColors[2] = new THREE.Color();
			}
		}

		//材質オブジェクトの宣言と生成
		var material = new THREE.MeshBasicMaterial({ color:0xFFFFFF, vertexColors:THREE.VertexColors});
		//２次元格子オブジェクトの生成
		this.CG = new THREE.Mesh( geometry, material );

	}

	//場の量の設定
	if( this.fieldValues.length > 0 ){

		//カラーマップの配色を更新
		this.updateColormap();

	} else {

		//２重配列の初期化
		for (var i = 0; i <= this.N; i++) {
			this.fieldValues[ i ] = [];
		}

	}

}

//テクスチャ生成メソッド
GRAPH.Colormap2D.prototype.generateCanvas = function( ) {

	//canvas要素の生成
	var canvas = document.createElement('canvas');
	//canvas要素のサイズ
	canvas.width = this.N;  //横幅
	canvas.height = this.N; //縦幅
	//コンテキストの取得
	var context = canvas.getContext('2d');

	//ビットマップデータのRGBAデータ格納配列
	var bitmapData = []; 
	//RGBAデータ格納配列への値の代入
	for (var i = 0; i < canvas.width; i++) {
		for (var j = 0; j < canvas.height; j++) {
			var index = (j * canvas.width + i) * 4; //各ピクセルの先頭を与えるインデクス番号

			if( this.fieldValues.length > 1 ){
				var color = this.getColor( i, j );
				var c = (this.max + this.min) / 2;
				color.a = Math.abs( 2 * (this.fieldValues[i][j] - c ) / ( this.max - this.min ) * this.transparentMode.factor);
				if( color.a > 1 ) color.a = 1;

			} else {

				var color = {r:1, g:1, b:1, a:1};

			}

			//ビットマップデータのRGBAデータ
			bitmapData[index + 0] = 255 * color.r; //R値
			bitmapData[index + 1] = 255 * color.g; //G値
			bitmapData[index + 2] = 255 * color.b; //B値
			bitmapData[index + 3] = 255 * color.a; //A値
		}
	}

	//イメージデータオブジェクトの生成
	var imageData = context.createImageData(canvas.width, canvas.height);
	for (var i = 0; i < canvas.width * canvas.height * 4; i++) {

		imageData.data[i] = bitmapData[i]; //配列のコピー

	}

	//イメージデータオブジェクトからcanvasに描画する
	context.putImageData(imageData, 0, 0);

	return canvas;
}


GRAPH.Colormap2D.prototype.updateColormap = function( ){

	//透過モード利用の有無
	if( this.transparentMode.enabled ){

		//テクスチャの再設定
		this.CG.material.map.image =  this.generateCanvas();
		//テクスチャの更新フラグ
		this.CG.material.map.needsUpdate = true;

	} else {

		for (var i = 0; i < this.N; i++) {

			for (var j = 0; j < this.N; j++) {

				//頂点番号
				var n = 2 * (this.N * i + j);

				//市松模様の暗部描画フラグ
				var darkFrag = false;

				//市松模様モードが指定されている場合
				if ( this.checkeredPatternMode.enabled && 
						( Math.round( i / this.checkeredPatternMode.width ) 
							+ Math.round( j / this.checkeredPatternMode.width ) ) % 2 == 0 ) darkFrag = true;

				//頂点座標データの更新
				this.CG.geometry.faces[n].vertexColors[0].copy( this.getColor( i, j, darkFrag  ) );
				this.CG.geometry.faces[n].vertexColors[1].copy( this.getColor( i + 1, j, darkFrag  ) );
				this.CG.geometry.faces[n].vertexColors[2].copy( this.getColor( i + 1, j + 1, darkFrag  ) );
				//頂点座標データの更新
				this.CG.geometry.faces[n+1].vertexColors[0].copy( this.getColor( i, j, darkFrag  ) );
				this.CG.geometry.faces[n+1].vertexColors[1].copy( this.getColor( i + 1, j + 1, darkFrag  ) );
				this.CG.geometry.faces[n+1].vertexColors[2].copy( this.getColor( i, j + 1, darkFrag  ) );

			}

		}

		//頂点色データ更新
		this.CG.geometry.colorsNeedUpdate = true;

	}

}

GRAPH.Colormap2D.prototype.getColor = function( i , j , darkFrag ){
	//場の量の取得
	var f = this.fieldValues[i][j];

	//カラーマップ関連プロパティ
	var p = this.colorParameter;
	if( p.HSLMode ){

		var value;
		if (f < this.min) value = 0;
		else if (f > this.max) value = this.max - this.min;
		else value = f - this.min;
		var H = p.H0 - ( value / (this.max - this.min) ) * p.Hw;

		//L値の取得
		var L = ( darkFrag )? this.checkeredPatternMode.L : p.L;

		//HSL形式で指定した色オブジェクトを返す
		return new THREE.Color( ).setHSL( H, p.S, L );

	} else {

		//描画範囲の中心値を取得
		var c = ( this.max + this.min ) / 2;
		//中心値よりも大きい場合
		var R = ( f > c )? ( f - c )/( this.max - c ) : 0;
		if( R > 1 ) R = 1;
		//中心値よりも小さい場合
		var B = ( f < c )? (c - f)/( c - this.min ) : 0;
		if( B > 1 ) B = 1;

		//中心値の色を白色とする場合
		if( p.backWhite ){
			var G = 1 - ( R + B );
			R = R + G;
			B = B + G;
		} else {
			var G = 0;
		}

		return new THREE.Color(R, G, B);

	}

}

GRAPH.Colormap2D.prototype.setValue = function( i , j, value ){

	this.fieldValues[i][j] = value || 0;

}

////////////////////////////////////////////////////////////////////
// ３次元グラフ描画クラスの定義
////////////////////////////////////////////////////////////////////
GRAPH.Graph3D = function ( parameter ) {
	//基底クラスのコンストラクタの実行
	GRAPH.Colormap2D.call( this, parameter );

	//ワイヤーフレーム描画の有無
	this.wireframe = parameter.wireframe || false;

	//不透明度
	this.opacity = parameter.opacity || 1;

	//両面描画の有無
	this.doubleSide = parameter.doubleSide || false;

	//描画色グラデーションの有無
	this.colorParameter.variable = ( parameter.colorParameter.variable !== undefined )? parameter.colorParameter.variable : true;

	//単一色描画の場合の描画色
	this.colorParameter.monocolor = parameter.colorParameter.monocolor || 0xFFFFFF;

	//３次元グラフのz成分の高さのスケール
	this.hightScale = parameter.hightScale || 0.2;

	//３次元グラフの初期化
	this.initGraph3D();
}
GRAPH.Graph3D.prototype = Object.create( GRAPH.Colormap2D.prototype );
GRAPH.Graph3D.prototype.constructor = GRAPH.Graph3D;

//３次元グラフオブジェクトの初期化
GRAPH.Graph3D.prototype.initGraph3D = function( ){

	//単一色描画の場合のHSL値の設定
	if( !this.colorParameter.variable ){
		//HSL形式で指定するフラグを立てる
		this.colorParameter.HSLMode = true;
		//H値の変化の傾きを0とする
		this.colorParameter.Hw = 0;

		//単一描画色のHSL値の取得
		var hsl = new THREE.Color( 
			this.colorParameter.monocolor 
		).getHSL();
		
		//HSL値の指定
		this.colorParameter.H = hsl.h;
		this.colorParameter.S = hsl.s;
		this.colorParameter.L = hsl.l;
		//２次元格子オブジェクトの描画色の更新
		this.updateColormap();
	}

	//両面の描画の有無
	if( this.doubleSide ) this.CG.material.side = THREE.DoubleSide;

	//ワイヤーフレーム表示の有無
	this.CG.material.wireframe = this.wireframe;

	//半透明の場合
	if( this.opacity < 1 ) {
		//透明化
		this.CG.material.transparent = true;
		//不透明度を指定
		this.CG.material.opacity = this.opacity;
	}

	//３次元グラフオブジェクトの更新
	this.updateGraph3D();
}

//３次元グラフオブジェクトの更新
GRAPH.Graph3D.prototype.updateGraph3D = function( ){

	//頂点番号
	var n = 0;
	for (var i = 0; i <= this.N; i++) {
		for (var j = 0; j <= this.N; j++) {

			//x方向とy方向の座標
			var nx = (-this.N / 2 + i) * this.l;
			var ny = (-this.N / 2 + j) * this.l;

			//場の量の取得
			var f = this.fieldValues[i][j];
			if( f > this.max ) f = this.max;
			if( f < this.min ) f = this.min;
			f = f - ( this.max + this.min ) / 2;

			var nz = f * this.hightScale;

			//頂点座標データの更新
			this.CG.geometry.vertices[ n ].set(nx, ny, nz); 

			n++;
		}
	}

	//座標点データ更新
	this.CG.geometry.verticesNeedUpdate = true;

	//半透明の場合
	if( this.opacity < 1 && !this.CG.material.transparent ) {

		//透明化
		this.CG.material.transparent = true;
		//不透明度を指定
		this.CG.material.opacity = this.opacity;

	} else if( this.opacity >= 1 && this.CG.material.transparent ){

		//透明化の解除
		this.CG.material.transparent = false;
		//不透明度を指定
		this.CG.material.opacity = 1;

	}

	//頂点色の更新
	if( this.colorParameter.variable ) this.updateColormap();
}


////////////////////////////////////////////////////////////////////
// ３次元カラーマップクラスの定義
////////////////////////////////////////////////////////////////////
GRAPH.Colormap3D = function( parameter ){

	//格子点数
	this.N = parameter.N || 100;
	//格子間隔
	this.l = parameter.l || 1.0 / this.N;

	//場の値（３重配列）
	this.fieldValues = parameter.fieldValues || [];

	//２次元カラーマップオブジェクトを格納するプロパティ
	this.xyPlane = null;
	this.yzPlane = null;
	this.zxPlane = null;

	//２次元カラーマップの更新を通知するフラグ
	this.updateXYplane = false;
	this.updateYZplane = false;
	this.updateZXplane = false;

	//３次元カラーマップオブジェクトの初期化
	this.initColormap3D( parameter );

}

//３次元カラーマップオブジェクトの初期化
GRAPH.Colormap3D.prototype.initColormap3D = function( parameter ){

	//場の量を初期化
	parameter.fieldValues = null;
	parameter.transparentMode = parameter.transparentMode || {};

	//yz平面、yz平面、zx平面の平面オブジェクトを生成
	this.xyPlane = new GRAPH.Colormap2D( parameter );
	this.yzPlane = new GRAPH.Colormap2D( parameter );
	this.zxPlane = new GRAPH.Colormap2D( parameter );

	if( parameter.transparentMode.enabled ){

		//xy平面とyz平面とzx平面を回転
		this.xyPlane.CG.rotation.set( Math.PI, 0, 0);
		this.yzPlane.CG.rotation.set( 0, Math.PI/2, Math.PI/2);
		this.zxPlane.CG.rotation.set( Math.PI/2, 0, Math.PI/2);

	} else {

		//yz平面とzx平面を回転
		this.yzPlane.CG.rotation.set( 0, Math.PI/2, Math.PI/2);
		this.zxPlane.CG.rotation.set( Math.PI/2, 0, -Math.PI/2);

	}


	//両面の描画の有無
	this.xyPlane.CG.material.side = THREE.DoubleSide;
	this.yzPlane.CG.material.side = THREE.DoubleSide;
	this.zxPlane.CG.material.side = THREE.DoubleSide;

	//マウスドラックによる移動軸の指定
	this.xyPlane.CG.allowMouseDragZaxis = true;
	this.yzPlane.CG.allowMouseDragXaxis = true;
	this.zxPlane.CG.allowMouseDragYaxis = true;


	//場の量の設定
	if( this.fieldValues.length > 0 ){

		//３次元カラーマップの配色を更新
		this.updateColormap( true, true, true );

	} else {

		//３重配列の初期化
		for (var i = 0; i <= this.N; i++) {

			this.fieldValues[ i ] = [];

			for (var j = 0; j <= this.N; j++) {

				this.fieldValues[ i ][ j ] = [];

			}

		}

	}

}

//３次元カラーマップの再描画
GRAPH.Colormap3D.prototype.updateColormap = function( updateXYplane, updateYZplane, updateZXplane ){

	//xy平面がドラックされた場合
	if( updateXYplane || this.updateXYplane || this.xyPlane.CG.mouseDraggingFlag ){
		//移動量の取得
		var kz = this.N/2 + Math.floor( this.xyPlane.CG.position.z / this.l );
		if( kz < 0 ) kz = 0;
		if( kz > this.N) kz = this.N;

		for (var i = 0; i <= this.N; i++) {
			for (var j = 0; j <= this.N; j++) {

				//場の量の取得
				this.xyPlane.fieldValues[i][j] = this.fieldValues[i][j][kz];

			}
		}
		//２次元カラーマップの更新
		this.xyPlane.updateColormap();

	}

	//yz平面がドラックされた場合
	if( updateYZplane || this.updateYZplane || this.yzPlane.CG.mouseDraggingFlag ){
		//移動量の取得
		var kx = this.N/2 + Math.floor( this.yzPlane.CG.position.x / this.l );
		if( kx < 0 ) kx = 0;
		if( kx > this.N) kx = this.N

		for (var i = 0; i <= this.N; i++) {
			for (var j = 0; j <= this.N; j++) {

				//場の量の取得
				this.yzPlane.fieldValues[i][j] = this.fieldValues[kx][i][j];

			}
		}
		//２次元カラーマップの更新
		this.yzPlane.updateColormap();

	}

	//zx平面がドラックされた場合
	if( updateZXplane || this.updateZXplane || this.zxPlane.CG.mouseDraggingFlag ){
		//移動量の取得
		var ky = this.N/2 + Math.floor( this.zxPlane.CG.position.y/ this.l );
		if( ky < 0 ) ky = 0;
		if( ky > this.N) ky = this.N;

		for (var i = 0; i <= this.N; i++) {
			for (var j = 0; j <= this.N; j++) {

				//場の量の取得
				this.zxPlane.fieldValues[i][j] = this.fieldValues[j][ky][i];

			}
		}
		//２次元カラーマップの更新
		this.zxPlane.updateColormap();

	}
	this.updateXYplane = false;
	this.updateYZplane = false;
	this.updateZXplane = false;

}

GRAPH.Colormap3D.prototype.setValue = function( i, j, k, value ){

	this.fieldValues[i][j][k] = value || 0;

}



////////////////////////////////////////////////////////////////////
// ２次元ベクトル場描画クラスの定義
////////////////////////////////////////////////////////////////////
GRAPH.VectorField2D = function( parameter ){
	parameter = parameter || {};

	//格子点数
	this.N = parameter.N || 20;
	//格子間隔
	this.l = parameter.l || 1.0 / this.N;
	//ベクトル量の大きさの最大値
	this.max = parameter.max || 1.0;

	//矢印関連パラメータ
	var ap = parameter.arrowParameter || {};
	this.arrowParameter = {
		variable : ap.variable || false,      //矢印の変化の有無
		maxLength : ( ap.maxLength !== undefined )? ap.maxLength : 1, //矢印の長さ最大値
		maxHeadLength : ( ap.maxHeadLength !== undefined )? ap.maxHeadLength : 0.3, //矢印頭部の長さの最大値
		maxHeadWidth : ( ap.maxHeadWidth !== undefined )? ap.maxHeadWidth : 0.4,    //矢印頭部の横幅の最大値
	};

	//矢印の色関連パラメータ
	var cp = parameter.colorParameter || {};
	this.colorParameter = {
		color : cp.color || 0xFFFFFF,   //矢印の色
		variable : ( cp.variable !== undefined )? cp.variable : true, //矢印の色の変化の有無
		gradation : cp.gradation || "White", //矢印色のグラデーション 「"White", "Black", "Red", "Blue", "Green"」
		backWhite : cp.backWhite || false //背景を白色を想定するフラグ
	};

	//２次元ベクトル場（２重配列）
	this.fieldVectors = parameter.fieldVectors || [];

	//コンピュータ・グラフィックスオブジェクト
	this.CG = null;

	//２次元ベクトル場の初期化
	this.initVectorField();
}

//２次元ベクトル場を実現するために必要な初期設定を行う
GRAPH.VectorField2D.prototype.initVectorField = function(){

	//矢印オブジェクトの親オブジェクトの生成
	this.CG = new THREE.Object3D( );

	//２次元格子点上に矢印オブジェクトを配置する
	for (var i = 0; i <= this.N; i++) {
		for (var j = 0; j <= this.N; j++) {

			var nx = (-this.N / 2 + i) * this.l;
			var ny = (-this.N / 2 + j) * this.l;
			var nz = 0;

			this.CG.add(
				new THREE.ArrowHelper(
					new THREE.Vector3( 1, 0, 0 ),    //方向
					new THREE.Vector3( nx, ny, nz ), //原点
					this.arrowParameter.maxLength * this.l,     //長さ
					this.colorParameter.color,                  //色
					this.arrowParameter.maxHeadLength * this.l, //矢頭の長さ
					this.arrowParameter.maxHeadWidth * this.l   //矢頭の幅
				)
			);
		}
	}

	//ベクトル場の量の設定
	if( this.fieldVectors.length > 0 ){

		//ベクトル場を更新
		this.updateVectorField();

	} else {

		//２重配列の初期化
		for (var i = 0; i <= this.N; i++) {
			
			this.fieldVectors[ i ] = [];

			for (var j = 0; j <= this.N; j++) {

				this.fieldVectors[ i ][ j ] = new THREE.Vector3(); 

			}

		}

	}

}

//場ベクトル場の量に対する矢印オブジェクトの更新を行う
GRAPH.VectorField2D.prototype.updateVectorField = function( ){

	//頂点番号
	var n = 0;
	for (var i = 0; i <= this.N; i++) {
		for (var j = 0; j <= this.N; j++) {

			//矢印オブジェクトの向きの指定
			this.CG.children[n].setDirection( 
				this.fieldVectors[i][j].clone().normalize() 
			);

			//矢印のサイズを変更する場合
			if( this.arrowParameter.variable ){

				//ベクトル場の量に対する矢印サイズのパラメータを取得
				var arrowParameter = this.getArrowParameter(i, j);

				//矢印オブジェクトのサイズを設定
				this.CG.children[n].setLength( 
					arrowParameter.lenght,
					arrowParameter.headLength,
					arrowParameter.headWidth					
				);

			}

			//矢印の色を変更する場合
			if( this.colorParameter.variable ){

				//ベクトル場の量に対する矢印の色のパラメータを取得
				var color = this.getColor(i ,j);

				//矢印オブジェクトの色を設定
				this.CG.children[n].setColor( color );

			} 

			n++;
		}

	}

}

//矢印オブジェクトのサイズを取得
GRAPH.VectorField2D.prototype.getArrowParameter = function( i, j ){

	//ベクトル場の大きさの取得
	var strength = this.fieldVectors[i][j].length();
	//ベクトル場の大きさの最大値との比較
	if( strength > this.max ) strength = this.max;
	//ベクトル場の大きさの規格化
	strength = strength / this.max;

	//矢印オブジェクトのサイズを格納するオブジェクトの宣言
	var arrowParameter = {};
	arrowParameter.lenght     = strength * this.arrowParameter.maxLength * this.l, 
	arrowParameter.headLength = strength * this.arrowParameter.maxHeadLength * this.l,
	arrowParameter.headWidth  = strength * this.arrowParameter.maxHeadWidth * this.l 

	return arrowParameter;
}

//矢印オブジェクトの色を取得
GRAPH.VectorField2D.prototype.getColor = function( i, j ){

	//ベクトル場の大きさの取得
	var strength = this.fieldVectors[i][j].length();
	//ベクトル場の大きさの最大値との比較
	if( strength > this.max ) strength = this.max;
	//ベクトル場の大きさの規格化
	strength = strength / this.max;

	//RGB値とする変数の宣言
	var R = 0;
	var G = 0;
	var B = 0;

	//グラデーション色に合わせてRGB値を指定
	if( this.colorParameter.gradation === "Red" ) R = strength;
	if( this.colorParameter.gradation === "Green" ) G = strength;
	if( this.colorParameter.gradation === "Blue" ) B = strength;

	//ゼロベクトルで白とする
	if( this.colorParameter.backWhite ){
		G = 1 - ( R + B );
		R = R + G;
		B = B + G;
	}

	//グラデーション色に合わせてRGB値を指定
	if( this.colorParameter.gradation === "White" ) R = G = B = strength;
	if( this.colorParameter.gradation === "Black" ) R = G = B = 1 - strength;

	return new THREE.Color(R, G, B);
}

//ベクトル場の量を設定
GRAPH.VectorField2D.prototype.setVector = function( i, j, vector ){

	this.fieldVectors[i][j].x = vector.x || 0;
	this.fieldVectors[i][j].y = vector.y || 0;
	this.fieldVectors[i][j].z = vector.z || 0;

}

////////////////////////////////////////////////////////////////////
// ３次元グラフプロットクラスの定義
////////////////////////////////////////////////////////////////////
GRAPH.Plot3D = function ( parameter ) {
	//基底クラスのコンストラクタの実行
	GRAPH.Graph3D.call( this, parameter );

	//軸オブジェクトを格納するプロパティ
	this.axis = {};
	//矢印オブジェクトの親オブジェクトの生成
	this.axis.CG  = new THREE.Object3D( );

	//線オブジェクトを格納するプロパティ
	this.lines = {};
	this.lines.ticsLength = parameter.ticsLength || 0.02; //目盛の長さ
	this.lines.hex = parameter.ticsColor || 0x888888;   //目盛の色
	this.lines.CG = new THREE.Object3D( );

	//初期化
	this.initPlot3D();
}
GRAPH.Plot3D.prototype = Object.create( GRAPH.Graph3D.prototype );
GRAPH.Plot3D.prototype.constructor = GRAPH.Plot3D;

//３次元グラフプロッターの描画を実現するために必要な初期設定を行う
GRAPH.Plot3D.prototype.initPlot3D = function(){

	//x軸方向矢印オブジェクトの生成と追加
	this.axis.CG.add(
		new THREE.ArrowHelper(
			new THREE.Vector3( 1, 0, 0 ), //方向
			new THREE.Vector3( 0, 0, 0 ), //原点
			1,             //長さ
			0xFF0000,      //色
			0.1,           //矢頭の長さ
			0.05           //矢頭の幅
		)
	);
	//y軸方向矢印オブジェクトの生成と追加
	this.axis.CG.add(
		new THREE.ArrowHelper(
			new THREE.Vector3( 0, 1, 0 ), //方向
			new THREE.Vector3( 0, 0, 0 ), //原点
			1,             //長さ
			0x00FF00,         //色
			0.1,         //矢頭の長さ
			0.05           //矢頭の幅
		)                  
	);
	//z軸方向矢印オブジェクトの生成と追加
	this.axis.CG.add(
		new THREE.ArrowHelper(
			new THREE.Vector3( 0, 0, 1 ), //方向
			new THREE.Vector3( 0, 0, 0 ), //原点
			1,             //長さ
			0x0000FF,         //色
			0.1,         //矢頭の長さ
			0.05           //矢頭の幅
		)                  
	);

	//位置を中心へ移動
	this.axis.CG.position.set(-0.5, -0.5, -0.5);

	//CGの子要素として追加
	this.CG.add( this.axis.CG );	

	//目盛用線オブジェクト生成関数
	function makeLine ( vertices, color, position ) {
		color = color || 0xFF0000;
		position = position || {x:0, y:0, z:0};


		//形状オブジェクトの宣言と生成
		var geometry = new THREE.Geometry( );
		//形状オブジェクトに頂点座標の設定
		for( var i = 0; i < vertices.length; i++ ){
			geometry.vertices.push( new THREE.Vector3( vertices[i].x, vertices[i].y, vertices[i].z ) );
		}

		//実線発光材質
		var material = new THREE.LineBasicMaterial( { color:color } );	

		//線オブジェクトの生成
		var line = new THREE.Line(geometry, material);
		line.position.set( position.x, position.y, position.z );

		return line;
	}

	//目盛の長さ
	var ticsLength = this.lines.ticsLength;
	//目盛の色
	var hex = this.lines.hex;

	//外枠
	this.lines.CG.add ( 
		makeLine ( 
			[ 
				{x:0, y:0, z:0},
				{x:1, y:0, z:0},
				{x:1, y:1, z:0},
				{x:0, y:1, z:0},
				{x:0, y:0, z:0}
			], 
			hex, 
			null 
		)
	);
	this.lines.CG.add ( 
		makeLine ( 
			[ 
				{x:0, y:0, z:1},
				{x:1, y:0, z:1},
				{x:1, y:1, z:1},
				{x:0, y:1, z:1},
				{x:0, y:0, z:1}
			], 
			hex, 
			null 
		)
	);
	this.lines.CG.add ( 
		makeLine ( 
			[ 
				{x:0, y:0, z:0},
				{x:0, y:0, z:1}
			], 
			hex, 
			null 
		)
	);		
	this.lines.CG.add ( 
		makeLine ( 
			[ 
				{x:0, y:1, z:0},
				{x:0, y:1, z:1}
			], 
			hex, 
			null 
		)
	);	
	this.lines.CG.add ( 
		makeLine ( 
			[ 
				{x:1, y:0, z:0},
				{x:1, y:0, z:1}
			], 
			hex, 
			null 
		)
	);
	this.lines.CG.add ( 
		makeLine ( 
			[ 
				{x:1, y:1, z:0},
				{x:1, y:1, z:1}
			], 
			hex, 
			null 
		)
	);

	//目盛用線オブジェクトの生成
	for( var i = 1; i <= 3; i++){

		//z=0の４つ
		this.lines.CG.add (
			makeLine ( 
				[ 
					{x:i*0.25, y:0, z: 0},
					{x:i*0.25, y:ticsLength, z: 0}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:i*0.25, y:1-ticsLength, z: 0},
					{x:i*0.25, y:1, z: 0}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:0, y:i*0.25, z:0},
					{x:ticsLength, y:i*0.25, z:0 }
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:1, y:i*0.25, z:0},
					{x:1-ticsLength, y:i*0.25, z:0}
				], 
				hex, 
				null 
			)
		);

		//z=1の４つ
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:i*0.25, y:0, z:1},
					{x:i*0.25, y:ticsLength, z:1}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:i*0.25, y:1-ticsLength, z: 1},
					{x:i*0.25, y:1, z: 1}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:ticsLength, y:i*0.25, z: 1},
					{x:0, y:i*0.25, z: 1}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:1-ticsLength, y:i*0.25, z: 1},
					{x:1, y:i*0.25, z: 1}
				], 
				hex, 
				null 
			)
		);
		//y=0の４つ
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:i*0.25, y:0, z: 0},
					{x:i*0.25, y:0, z: ticsLength}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:i*0.25, y:0, z: 1},
					{x:i*0.25, y:0, z: 1-ticsLength}
				], 
				hex, 
				null 
			)
		);

		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:0, y:0, z:i*0.25},
					{x:ticsLength, y:0, z: i*0.25}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:1, y:0, z:i*0.25},
					{x:1-ticsLength, y:0, z: i*0.25}
				], 
				hex, 
				null 
			)
		);
		//y=1の４つ
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:i*0.25, y:1, z: 1- ticsLength},
					{x:i*0.25, y:1, z: 1}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:i*0.25, y:1, z: 0},
					{x:i*0.25, y:1, z: ticsLength}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:0, y:1, z:i*0.25},
					{x:ticsLength, y:1, z: i*0.25}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:1-ticsLength, y:1, z:i*0.25},
					{x:1, y:1, z: i*0.25}
				], 
				hex, 
				null 
			)
		);


		//x=0の４つ

		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:0, y:1-ticsLength, z:i*0.25},
					{x:0, y:1, z: i*0.25}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:0, y:i*0.25, z:0},
					{x:0, y:i*0.25, z: ticsLength}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:0, y:i*0.25, z: 1-ticsLength},
					{x:0, y:i*0.25, z: 1}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:0, y:0, z:i*0.25},
					{x:0, y:ticsLength, z: i*0.25}
				], 
				hex, 
				null 
			)
		);

		//x=1の２つ
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:1, y:i*0.25, z:0},
					{x:1, y:i*0.25, z: ticsLength}
				], 
				hex, 
				null 
			)
		);
		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:1, y:i*0.25, z: 1-ticsLength},
					{x:1, y:i*0.25, z: 1}
				], 
				hex, 
				null 
			)
		);

		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:1, y:0, z:i*0.25},
					{x:1, y:ticsLength, z: i*0.25}
				], 
				hex, 
				null 
			)
		);


		this.lines.CG.add ( 
			makeLine ( 
				[ 
					{x:1, y:1-ticsLength, z:i*0.25},
					{x:1, y:1, z: i*0.25}
				], 
				hex, 
				null 
			)
		);

	}

	//位置を中心へ移動
	this.lines.CG.position.set(-0.5, -0.5, -0.5);

	//CGの子要素として追加
	this.CG.add( this.lines.CG );

}

GRAPH.Plot3D.prototype.updatePlot3D = function(){


	this.updateGraph3D();

}




/////////////////////////////////////////////////////////////////////////




	/////////////////////////////////////////////////////////
	//画面キャプチャ生成
	/////////////////////////////////////////////////////////
	function makePicture(){
		if( picture.makeFlag && usePicture ){
			//canvas要素→DataURL形式
			document.getElementById( picture.downloadID ).href = renderer.domElement.toDataURL("image/png");
			//PNGファイル名の命名
			document.getElementById( picture.downloadID ).download = picture.filename;

			picture.makeFlag = false;
		}
	}
	////////////////////////////////////////////////////////////////////
	// ビデオ生成
	////////////////////////////////////////////////////////////////////
	function makeVideo(){

		//録画中フラグのチェック
		if( video.recordingFlag && useVideo ){
			//動画フレームの追加
			video.add( renderer.domElement );

		} else if( video.compileFlag ){

			//動画フレームの存在のチェック
			if( video.frames.length > 0 ){

				//動画コンパイル前時刻の取得
				var start_time = new Date;

				//動画コンパイル
				var blob = video.compile();

				//動画コンパイル後時刻の取得
				var end_time = new Date;

				//動画コンパイル時間の計算
				var time = end_time - start_time;

				//BlobURLを取得しa要素のsrcへ与える
				document.getElementById( video.downloadID ).href = window.URL.createObjectURL( blob );
				//ファイル名を指定
				document.getElementById( video.downloadID ).download = video.filename;

				//動画情報の可視化
				console.log( "ファイルサイズ：" + Math.ceil( blob.size / 1024 ) + "[KB]  コンパイル時間：" + time + "[ms]" );

				//ダウンロード用a要素の表示
				document.getElementById( video.downloadID ).style.display = "block";

				//動画フレームの初期化
				video.frames = [];

			}

			//動画コンパイル終了
			video.compileFlag = false;

		}

	}
	////////////////////////////////////////////////////////////////////
	// マウスドラック準備関数
	////////////////////////////////////////////////////////////////////
	var rayReceiveObjects = []; //光線を受けるオブジェクト配列
	function initDragg(){

		///////////////光線受信用平面オブジェクトの定義//////////////////
		//形状オブジェクトの宣言と生成
		var geometry = new THREE.PlaneBufferGeometry(20, 20, 8, 8);
		//材質オブジェクトの宣言と生成
		var material = new THREE.MeshBasicMaterial({ color: 0x000000,  wireframe: true })
		//平面オブジェクトの生成
		var plane = new THREE.Mesh( geometry, material );
		//平面オブジェクトの可視化
		plane.visible = false;
		//平面オブジェクトのシーンへの追加
		scene.add(plane);

		canvasFrame.addEventListener('mousemove', onDocumentMouseMove, false);//
		canvasFrame.addEventListener('mousedown', onDocumentMouseDown, true); //<-trueに変更
		canvasFrame.addEventListener('mouseup'  , onDocumentMouseUp,   false);//
		canvasFrame.addEventListener('mouseout' , onDocumentMouseUp, false);  //<--追記（額縁外にマウスドラック時の修正）

		//マウスクリック時の選択したオブジェクト中心からのマウスポインタのズレ
		var offset = new THREE.Vector3();
		var INTERSECTED; //マウスポインタが指しているオブジェクト 
		var SELECTED;    //マウスドラック中のオブジェクト

		//HTML要素の位置による補正量の取得
		var elementOffsetLeft, elementOffsetTop; 
		//マウスポインタの位置
		var mouse = new THREE.Vector2();
		//光線発射オブジェクト
		var raycaster = new THREE.Raycaster();

		function onDocumentMouseMove(event) {

			//canvas要素の絶対座標の取得
			elementOffsetLeft = canvasFrame.getBoundingClientRect( ).left; 
			elementOffsetTop = canvasFrame.getBoundingClientRect( ).top; 

			//クリップ座標系におけるマウスポインタの位置座標の取得
			mouse.x = ( (event.clientX-elementOffsetLeft) / canvasFrame.clientWidth) * 2 - 1;
			mouse.y = -( (event.clientY-elementOffsetTop) / canvasFrame.clientHeight) * 2 + 1;

			//マウスポインタの位置をと現時点のカメラ関連パラメータを設定
			raycaster.setFromCamera( mouse, camera );

			//オブジェクトがマウスドラックされている時
			if (SELECTED) {
				//光線と交わる平面オブジェクトオブジェクトを収集
				var intersects = raycaster.intersectObject(plane);
				//マウスドラック時のマウスポインタの指している平面オブジェクトの３次元空間中の位置座標
				var vec3 = intersects[0].point;
				var v = vec3.sub( offset );

				//マウスドラックされているオブジェクトを移動
				if( SELECTED.allowMouseDragXaxis ) SELECTED.position.x = v.x;
				if( SELECTED.allowMouseDragYaxis ) SELECTED.position.y = v.y;
				if( SELECTED.allowMouseDragZaxis ) SELECTED.position.z = v.z;
				if( !(SELECTED.allowMouseDragXaxis || SELECTED.allowMouseDragYaxis || SELECTED.allowMouseDragZaxis) ){
					SELECTED.position.copy( v );
				}
				//マウスドラック中フラグの設定
				SELECTED.mouseDraggingFlag = true;
				return;
			}

			//光線と交わるオブジェクトを収集
			var intersects = raycaster.intersectObjects( rayReceiveObjects )
			//マウスポインタがオブジェクト上にある場合
			if (intersects.length > 0) {
				if (INTERSECTED != intersects[0].object) {
					//マウスポインタが指しているオブジェクトが登録されていなければ、一番手前のオブジェクトを「INTERSECTED」に登録
					INTERSECTED = intersects[0].object;
					//平面オブジェクトの位置座標を「INTERSECTED」に登録されたオブジェクトと同じ位置座標とする
					plane.position.copy(INTERSECTED.position);
					//平面オブジェクトの上ベクトルをカメラの位置座標の方向へ向ける
					plane.lookAt(camera.position);
				}
				//マウスポインタのカーソルを変更
				canvasFrame.style.cursor = 'pointer';
			} else {
				//マウスポインタがオブジェクトから離れている場合
				INTERSECTED = null;
				//マウスポインタのカーソルを変更
				canvasFrame.style.cursor = 'auto';
			}
		}

		function onDocumentMouseDown(event) {

			//光線と交わるオブジェクトを収集
			var intersects = raycaster.intersectObjects( rayReceiveObjects );

			//交わるオブジェクトが１個以上の場合
			if (intersects.length > 0) {
				//トラックボールを無効化
				trackball.enabled = false;
				//クリックされたオブジェクトを「SELECTED」に登録
				SELECTED = intersects[0].object;

				//光線と交わる平面オブジェクトオブジェクトを収集
				var intersects = raycaster.intersectObject(plane);
				//クリック時のマウスポインタの指した平面オブジェクトの３次元空間中の位置座標
				var vec3 = intersects[0].point;
				//平面オブジェクトの中心から見た相対的な位置座標
				offset.copy(vec3).sub(plane.position);
				//マウスポインタのカーソルを変更
				canvasFrame.style.cursor = 'move';
			}
		}
		function onDocumentMouseUp(event) {
			//トラックボールを有効化
			trackball.enabled = true;

			//画面キャプチャの生成
			picture.makeFlag = true;

			//マウスアップ時にマウスポインタがオブジェクト上にある場合
			if (INTERSECTED) {
				//マウスドラック中フラグの解除
				SELECTED.mouseDraggingFlag = false;
				//平面オブジェクトの位置座標をオブジェクトの位置座標に合わせる
				plane.position.copy(INTERSECTED.position);
				//マウスドラックの解除
				SELECTED = null;
			}
			//マウスポインタのカーソルを変更
			canvasFrame.style.cursor = 'auto';
		}
	}